//
//  RxRequest.swift
//  flutter_stripe
//
//  Created by Ricky Lau on 31/7/2018.
//

import RxSwift
import RxAlamofire
import Alamofire
import ObjectMapper

class RxRequest {
    static var domain: String {
        return "http://ticket-api-v2-dev.hkmovie6.com"
    }
    
    public static func submitAction(_ urlRequest: URLRequestConvertible, sessionManager: SessionManager = SessionManager.default) -> Observable<Void> {
        return sessionManager.rx.request(urlRequest: urlRequest)
            .flatMap{ $0.rx.jsonWithErrorData() }
            .map{ _ in () }
            .observeOn(MainScheduler.instance)
    }
    
    static func requestObject<T: BaseMappable>(_ urlRequest: URLRequestConvertible, type: T.Type , sessionManager: SessionManager = SessionManager.default) -> Observable<T> {
        return sessionManager.rx.request(urlRequest: urlRequest)
            .flatMap { $0.rx.jsonWithErrorData() }
            .flatMap { RxRequest.mapObject(type, data: $0) }
            .observeOn(MainScheduler.instance)
    }
    
    static func requestArray<T: BaseMappable>(_ urlRequest: URLRequestConvertible, type: T.Type , sessionManager: SessionManager = SessionManager.default) -> Observable<[T]> {
        return sessionManager.rx.request(urlRequest: urlRequest)
            .flatMap { $0.rx.jsonWithErrorData() }
            .flatMap { RxRequest.mapJsonArray(type, data: $0) }
            .observeOn(MainScheduler.instance)
    }
    
    fileprivate static func mapObject<T: BaseMappable>(_ type: T.Type, data: Any) -> Observable<T> {
        return Observable.create { observer in
            if let m: T = Mapper<T>().map(JSONObject: data){
                observer.onNext(m)
                observer.onCompleted()
            } else {
                observer.onError(RxRequestError.ObjectMappingError())
            }
            return Disposables.create{ }
        }
    }
    
    fileprivate static func mapJsonArray<T: BaseMappable>(_ type: T.Type, data: Any) -> Observable<[T]> {
        return Observable.create { observer in
            if let m: [T] = Mapper<T>().mapArray(JSONObject: data){
                observer.onNext(m)
                observer.onCompleted()
            } else {
                observer.onError(RxRequestError.ObjectMappingError())
            }
            return Disposables.create{ }
        }
    }
}

public extension Reactive where Base: DataRequest {
    
    // Have to do this because validateSuccessfulResponse() in RxAlamofire has `internal` visibility
    public func validateSuccessfulAndFailedResponse() -> DataRequest {
        return self.base.validate(statusCode: 200 ..< 300)
    }
    
    public func jsonWithErrorData(options: JSONSerialization.ReadingOptions = .allowFragments) -> Observable<Any> {
        return resultWithErrorData(responseSerializer: Base.jsonResponseSerializer(options: options))
    }
    
    public func resultWithErrorData<T: DataResponseSerializerProtocol>(queue: DispatchQueue? = nil, responseSerializer: T) -> Observable<T.SerializedObject> {
        
        return Observable.create { observer in
            let dataRequest = self.validateSuccessfulAndFailedResponse()
                .response(queue: queue, responseSerializer: responseSerializer) { (packedResponse) -> Void in
                    switch packedResponse.result {
                    case .success(let result):
                        if let _ = packedResponse.response {
                            observer.on(.next(result))
                            observer.on(.completed)
                        } else {
                            observer.on(.error(RxAlamofireUnknownError))
                        }
                        
                    case .failure(let error):
                        let body = String(data: packedResponse.data!, encoding: String.Encoding.utf8) ?? ""
                        let serverError = ServerError(JSONString: body) ?? ServerError(error)
                        observer.on(.error(RxRequestError.NetworkError(serverError)))
                    }
            }
            return Disposables.create {
                dataRequest.cancel()
            }
        }
    }
}

