//
//  ServerError.swift
//  flutter_stripe
//
//  Created by Ricky Lau on 31/7/2018.
//

import ObjectMapper

enum RxRequestError: Error {
    case NetworkError(ServerError)
    case ObjectMappingError()
    
    var localizedDescription: String {
        switch self {
        case .ObjectMappingError():
            return "Object mapping error"
        case .NetworkError(let error):
            return error.readableMessage
        }
    }
}

class ServerError: Mappable {
    
    public var timestamp: String = ""
    public var message: String = "Unknown Error"
    public var type: String = "Unknown Error"
    public var code: Int = 0
    
    init(_ error: Error) {
        message = error.localizedDescription
    }
    
    required init?(map: Map) {}
    
    func mapping(map: Map) {
        timestamp <- map["timestamp"]
        message <- map["message"]
        type <- map["type"]
        code <- map["code"]
    }
    
    var readableMessage: String {
        return """
        [\(code)] - \(type.capitalizingFirstLetter())
        \(message.capitalizingFirstLetter())
        """
    }
}
